"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.GithubFetcher = void 0;

var _core = require("@yarnpkg/core");

var _fslib = require("@yarnpkg/fslib");

var _tmp = require("tmp");

var githubUtils = _interopRequireWildcard(require("./githubUtils"));

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

class GithubFetcher {
  supports(locator, opts) {
    if (!githubUtils.isGithubUrl(locator.reference)) return false;
    return true;
  }

  getLocalPath(locator, opts) {
    return null;
  }

  async fetch(locator, opts) {
    const expectedChecksum = opts.checksums.get(locator.locatorHash) || null;
    const [packageFs, releaseFs, checksum] = await opts.cache.fetchPackageFromCache(locator, expectedChecksum, async () => {
      opts.report.reportInfoOnce(_core.MessageName.FETCH_NOT_CACHED, `${_core.structUtils.prettyLocator(opts.project.configuration, locator)} can't be found in the cache and will be fetched from the remote repository`);
      return await this.fetchFromNetwork(locator, opts);
    });
    return {
      packageFs,
      releaseFs,
      prefixPath: _core.structUtils.getIdentVendorPath(locator),
      checksum
    };
  }

  async fetchFromNetwork(locator, opts) {
    const sourceBuffer = await _core.httpUtils.get(this.getLocatorUrl(locator, opts), {
      configuration: opts.project.configuration
    });

    const extractPath = _fslib.npath.toPortablePath((0, _tmp.tmpNameSync)());

    const extractTarget = new _fslib.CwdFS(extractPath);
    await _core.tgzUtils.extractArchiveTo(sourceBuffer, extractTarget, {
      stripComponents: 1
    });

    const packagePath = _fslib.ppath.join(extractPath, `package.tgz`);

    await _core.scriptUtils.prepareExternalProject(extractPath, packagePath, {
      configuration: opts.project.configuration,
      report: opts.report
    });
    const packedBuffer = await _fslib.xfs.readFilePromise(packagePath);
    return await _core.tgzUtils.convertToZip(packedBuffer, {
      stripComponents: 1,
      prefixPath: _core.structUtils.getIdentVendorPath(locator)
    });
  }

  getLocatorUrl(locator, opts) {
    const {
      auth,
      username,
      reponame,
      treeish
    } = githubUtils.parseGithubUrl(locator.reference);
    return `https://${auth ? `${auth}@` : ``}github.com/${username}/${reponame}/archive/${treeish}.tar.gz`;
  }

}

exports.GithubFetcher = GithubFetcher;